#-*- coding: utf-8 -*-

from bgesdk.error import APIError

import pytest


class TestUser:

    def test_get_user(self, oauth2):
        """无效 access_token 调用接口报错"""
        api = oauth2.get_api('demo')
        with pytest.raises(APIError) as e:
            api.get_user()
        assert e.value.code == 403
        assert e.value.msg == u'access_token 已失效或错误'



#-*- coding: utf-8 -*-

from bgesdk.client import OAuth2

import logging
import pytest


base_url = 'https://api.bge.genomics.cn'
client_id = 'ex8FoZjiSmjAlbg0XiqGmaTSIT3SGxoJYnslDGqB'      # XXX change this for test
client_secret = 'mlPmKpAu4yjglBH070tWwBDnT6eoT7c0zwY0uaTsZ9h9xlPVcumCgeAg0h7NvT6dm286AIisgVl6utaDwkj0Ge3qBb8MZJN8nT5oVGzbB8SEqZocBSu1N5qcOAIjmBdT'  # XXX change this for test


@pytest.fixture(scope='session')
def redirect_url():
    return 'http://test.cn'  # XXX change this for test


@pytest.fixture(scope='session')
def logger():
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    return logger


@pytest.fixture(scope='session')
def oauth2():
    return OAuth2(client_id, client_secret, base_url=base_url)


@pytest.fixture(scope='session')
def access_token():
    access_token = 'BbKcYlxf3otNqtomr4c9zyJlyehfd5'  # XXX change this for test
    return access_token


@pytest.fixture(scope='session')
def api(oauth2, access_token):
    return oauth2.get_api(access_token)


@pytest.fixture(scope='session')
def self_biosample_id():
    """本人的样品编号"""
    return 'E-B19941661351'  # XXX change this for test


@pytest.fixture(scope='session')
def other_biosample_id():
    """其他人的样品编号"""
    return 'E-B21825354978'  # XXX change this for test



#-*- coding: utf-8 -*-

from . import contants
from .error import APIError, BGEError
from .utils import major_version

from requests.adapters import HTTPAdapter
if major_version <= 2:
    from urlparse import urljoin
else:
    from urllib.parse import urljoin

import logging
import requests

logger = logging.getLogger()
DEFAULT_TIMEOUT = contants.HTTP_DEFAULT_TIMEOUT
MAX_RETRIES = contants.HTTP_MAX_RETRIES
HTTP_SESSION = requests.Session()
HTTP_SESSION.mount('http://', HTTPAdapter(max_retries=MAX_RETRIES))
HTTP_SESSION.mount('https://', HTTPAdapter(max_retries=MAX_RETRIES))


class HTTPRequest(object):
    """HTTP 请求类

    处理 SDK 接口的 HTTP 调用；

    Args:
        base_url (str): 平台基本地址，如 https://api.bge.genomics.cn；
    """

    def __init__(self, base_url):
        self.base_url = base_url
        self.headers = {}

    def set_authorization(self, access_token):
        """设置 Authorization 头部

        Args:
            access_token (str): 访问令牌
        """
        self.headers['Authorization'] = 'Bearer {}'.format(access_token)

    def get(self, path, params=None, timeout=None):
        """GET 接口请求

        Args:
            path (str): 请求路径
            params (dict, 非必填): GET 参数；
            timeout (int, 非必填): 请求超时时间；

        Returns:
            object: 请求返回值
        """
        return self._request('GET', path, params=params, timeout=timeout)

    def post(self, path, params=None, data=None, timeout=None):
        """POST

        Args:
            path (str): 请求路径
            params (dict, 非必填): GET 参数；
            data (dict, 非必填): POST 参数；
            timeout (int, 非必填): 请求超时时间；

        Returns:
            object: 请求返回值
        """
        return self._request(
            'POST', path, params=params, data=data, timeout=timeout)

    def _request(self, method, path, **kwargs):
        """发送 HTTP 请求

        Args:
            method (str): 请求方法
            path (str): 请求路径

        Raises:
            BGEError: SDK 错误
            APIError: API 错误

        Returns:
            object: 返回值
        """
        headers = self.headers
        url = urljoin(self.base_url, path)
        logger.debug('%s %s %s', method, url, kwargs)
        timeout = kwargs.get('timeout')
        if timeout is None:
            kwargs['timeout'] = DEFAULT_TIMEOUT
        resp = HTTP_SESSION.request(
            method=method, url=url, headers=headers, **kwargs)
        try:
            resp.raise_for_status()
        except requests.exceptions.HTTPError as e:
            raise BGEError('API request error: %s' % e)
        content_type_header = resp.headers.get('Content-Type', '')
        if 'application/json' not in content_type_header:
            return resp.text
        result = resp.json()
        data = result.get('data')
        code = result['code']
        msg = result['msg']
        if code != 0:
            raise APIError(code, msg, data)
        pagination = result.get('pagination')
        if pagination:
            # 相关接口分页返回方式升级后，可去除此处
            # TODO: upgrade in the future
            return data, pagination
        return data



#-*- coding: utf-8 -*-

from bgesdk.error import APIError

import pytest


class TestSample:

    def check_samples(self, result):
        assert 'result' in result
        assert 'count' in result
        assert 'next_page' in result
        next_page = result['next_page']
        assert isinstance(result['result'], list)
        assert isinstance(result['count'], int)
        assert isinstance(next_page, int) or next_page is None

    @pytest.mark.parametrize('biosample_sites', range(1, 16))
    def test_valid_biosample_sites(self, api, biosample_sites):
        """在参数 choices 范围限制内"""
        ret = api.get_samples(biosample_sites=biosample_sites)
        self.check_samples(ret)

    @pytest.mark.parametrize('biosample_sites', [0, '0,3', 20, '3,20', 'x'])
    def test_invalid_biosample_sites(self, api, biosample_sites):
        """超出参数 choices 范围限制"""
        with pytest.raises(APIError) as e:
            api.get_samples(biosample_sites=biosample_sites)
        assert e.value.code == 41001
        assert e.value.msg == u'参数错误'

    @pytest.mark.parametrize('omics', range(1, 3))
    def test_valid_omics(self, api, omics):
        """在参数 choices 范围限制内"""
        ret = api.get_samples(omics=omics)
        self.check_samples(ret)

    @pytest.mark.parametrize('omics', [0, '0,3', 4, '3,4', 'x'])
    def test_invalid_omics(self, api, omics):
        """超出参数 choices 范围限制"""
        with pytest.raises(APIError) as e:
            api.get_samples(omics=omics)
        assert e.value.code == 41001
        assert e.value.msg == u'参数错误'

    @pytest.mark.parametrize('organisms', range(1, 3))
    def test_valid_organisms(self, api, organisms):
        """在参数 choices 范围限制内"""
        ret = api.get_samples(organisms=organisms)
        self.check_samples(ret)

    @pytest.mark.parametrize('organisms', [0, '0,3', 4, '3,4', 'x'])
    def test_invalid_organisms(self, api, organisms):
        """超出参数 choices 范围限制"""
        with pytest.raises(APIError) as e:
            api.get_samples(organisms=organisms)
        assert e.value.code == 41001
        assert e.value.msg == u'参数错误'

    def test_result(self, api, logger):
        """返回值"""
        ret = api.get_samples(limit=1)
        self.check_samples(ret)
        next_page = ret['next_page']
        if next_page is not None:
            ret = api.get_samples(page=next_page)
            self.check_samples(ret)



#-*- coding: utf-8 -*-

import json


class BGEError(Exception):
    """SDK 错误"""


class APIError(BGEError):
    """接口错误"""

    def __init__(self, code, msg, data=None):
        self.code = int(code)
        self.msg = msg
        self.data = data
        super(BGEError, self).__init__(json.dumps({
            'code': code,
            'msg': msg,
            'data': data
        }, indent=4, ensure_ascii=False))


#-*- coding: utf-8 -*-

from bgesdk.error import APIError
from bgesdk.utils import major_version

import pytest


def check_result(result):
    assert 'result' in result
    assert 'count' in result
    assert 'next_page' in result
    next_page = result['next_page']
    assert isinstance(result['result'], list)
    assert isinstance(result['count'], int)
    assert isinstance(next_page, int) or next_page is None


class TestTaxonAbundance:

    @pytest.mark.parametrize('taxon_ids', [None, 'tx1', 'tx2'])
    def test_result(self, api, logger, self_biosample_id, taxon_ids):
        """正常返回的数据"""
        ret = api.get_taxon_abundance(self_biosample_id)
        logger.debug(ret)
        check_result(ret)

    @pytest.mark.parametrize('taxon_ids', ['txdemo', 'tx', 'test'])
    def test_invalid_txid(self, api, logger, self_biosample_id, taxon_ids):
        """格式错误的 taxon 编号"""
        ret = api.get_taxon_abundance(self_biosample_id)
        logger.debug(ret)
        check_result(ret)


class TestFuncAbundance:

    @pytest.mark.parametrize('catalog', ['go', 'ko', 'eggnog', 'pfam',
                                         'kegg-pwy', 'kegg-mdl', 'level4ec',
                                         'metacyc-rxn', 'metacyc-pwy'])
    def test_result(self, api, logger, self_biosample_id, catalog):
        """正常返回的数据"""
        try:
            ret = api.get_func_abundance(self_biosample_id, catalog)
        except APIError as error:
            with pytest.raises(APIError) as e:
                raise error
            e.value.code == 41202
            e.value.msg == u'BGE 私有接口错误: 样品数据未入仓'
            return
        logger.debug(ret)
        check_result(ret)


class TestGeneAbundance:

    def check_result(self, result):
        assert 'result' in result
        assert 'count' in result
        assert 'next_page' in result
        next_page = result['next_page']
        assert isinstance(result['result'], list)
        assert isinstance(result['count'], int)
        str_ = str
        if major_version <= 2:
            str_ = unicode
        assert isinstance(next_page, str_) or next_page is None

    @pytest.mark.parametrize('catalog, data_type', [
        ('IGC_9.9M', 'list'),
        ('UniRef90_HUMAnN2_0.11', 'file')])
    def test_result(self, api, logger, self_biosample_id, catalog,
                    data_type):
        """正常返回的数据"""
        ret = api.get_gene_abundance(self_biosample_id, catalog, data_type)
        logger.debug(ret)
        self.check_result(ret)

    @pytest.mark.parametrize('catalog, data_type', [
        ('IGC_9.9M', 'file'),
        ('UniRef90_HUMAnN2_0.11', 'list')])
    def test_invalid_args(self, api, self_biosample_id, catalog, data_type):
        """正常返回的数据"""
        with pytest.raises(APIError) as e:
            api.get_gene_abundance(self_biosample_id, catalog, data_type)
        assert e.value.code == 41001
        assert e.value.msg == u'参数错误'



#-*- coding: utf-8 -*-

from .utils import major_version

if major_version <= 2:
    from UserDict import UserDict
else:
    from collections import UserDict
from weakref import proxy

import json


class Model(UserDict):
    """统一的接口返回数据包装模型类

    所有接口的返回值都使用本类进行封装。

    使用示例:

        >>> print(Model({}))
        Model({})
        >>> inst = Model({})
        >>> inst['demo'] = 1
        >>> print(inst.demo)
        1
        >>> print(inst['demo'])
        1
        >>> demo = {
            'x': [{
                'x1': 'x1'
            }],
            'y': {
                'y1': 'y1'
            }
        }
        >>> m = Model(demo)
        >>> print(m)
        Model({'y': {'y1': 'y1'}, 'x': [{'x1': 'x1'}]})
        >>> j = m.json()
        >>> print(type(j))
        <type 'dict'>
        >>> print(j)
        {'y': {'y1': 'y1'}, 'x': [{'x1': 'x1'}]}
        >>> s = m.dumps(indent=4)
        >>> print(type(s))
        <type 'str'>
        >>> print(s)
        {
            "y": {
                "y1": "y1"
            }, 
            "x": [
                {
                    "x1": "x1"
                }
            ]
        }
    """

    def __init__(self, data):
        UserDict.__init__(self, data)
        ret = {}
        for key, val in self.data.items():
            ret[key] = self._encode_data(val)

    def __getattr__(self, name):
        try:
            return self.data[name]
        except KeyError:
            raise AttributeError(name)

    def json(self):
        """返回可 JSON 序列化的对象"""
        ret = {}
        for key, value in self.data.items():
            ret[key] = self._decode_data(value)
        return ret

    def dumps(self, indent=None, ensure_ascii=None, **kwargs):
        r = self.json()
        return json.dumps(
            r, indent=indent, ensure_ascii=ensure_ascii, **kwargs)

    def _encode_data(self, val):
        if isinstance(val, dict):
            ret = {}
            for sub_key, sub_val in val.items():
                ret[sub_key] = self._encode_data(sub_val)
            return self.__class__(ret)
        elif isinstance(val, list):
            ret = []
            for sub_val in val:
                ret.append(self._encode_data(sub_val))
            return ret
        else:
            return val

    def _decode_data(self, val):
        if isinstance(val, Model):
            ret = {}
            for sub_key, sub_val in val.items():
                ret[sub_key] = self._decode_data(sub_val)
            return ret
        elif isinstance(val, list):
            ret = []
            for sub_val in val:
                ret.append(self._decode_data(sub_val))
            return ret
        else:
            return val

    def __str__(self):
        return '%s(%s)' % (self.__class__.__name__, self.data)

    def __repr__(self):
        return '<%s object at %s>' % (
            self.__class__.__name__, hex(id(self)))


class AuthorizationCodeToken(Model):
    """授权码模式返回的令牌对象"""

    def __init__(self, oauth2, data):
        Model.__init__(self, data)
        self._oauth2 = proxy(oauth2)

    def refresh(self):
        """刷新授权码模式访问令牌"""
        oauth2 = self._oauth2
        refresh_token = self.data['refresh_token']
        return oauth2.exchange_refresh_token(refresh_token)


class ClientCredentialsToken(Model):
    """客户端模式返回的令牌对象"""

    def __init__(self, oauth2, data):
        Model.__init__(self, data)
        self._oauth2 = proxy(oauth2)

    def refresh(self):
        """刷新（重新获取）客户端模式访问令牌"""
        oauth2 = self._oauth2
        return oauth2.get_credentials_token()



#-*- coding: utf-8 -*-

from bgesdk.error import APIError

import pytest


class TestModel:

    @pytest.mark.parametrize('model_id', ['unknown_model_id'])
    def test_unknown_model(self, api, logger, model_id):
        """格式错误的 taxon 编号"""
        with pytest.raises(APIError) as e:
            api.invoke_model(model_id)
        assert e.value.code == 41202
        assert e.value.msg.startswith(u'BGE 私有接口错误: 模型不存在')



#-*- coding: utf-8 -*-

"""
BGE 开放平台 SDK 客户端模块。

当前模块提供了两个主要的类 OAuth2 和 API，他们分别提供了对 BGE 开放平台的 OAuth2 相关
接口和其他接口的调用方法。

使用示例：

    >>> oauth2 = OAuth2('demo', 'demo')
    >>> token = oauth2.get_credentials_token()
    >>> api = oauth2.get_api(token.access_token)
    >>> api.invoke_model('demo_model_id')
    Model({...})
"""

from . import contants
from . import models
from .error import BGEError
from .http import HTTPRequest
from .utils import major_version

if major_version <= 2:
    from urlparse import urljoin
    from urllib import urlencode
else:
    from urllib.parse import urljoin, urlencode
from weakref import proxy

import oss2


BASE_URL = contants.BASE_URL
DEFAULT_TIMEOUT = contants.HTTP_DEFAULT_TIMEOUT
GRANT_AUTHORIZATION_CODE = contants.GRANT_AUTHORIZATION_CODE
GRANT_TYPE_CREDENTIALS = contants.GRANT_TYPE_CREDENTIALS

__all__ = ['OAuth2', 'API']


class OAuth2(object):
    """OAuth2 授权客户端类。
    
    管理关于 OAuth2 相关接口的调用，包括获取授权页面地址、授权码交换访问令牌等；
    
    Args:
        client_id (字符串): 第三方客户端 client_id；
        client_secret (字符串): 第三方客户端 client_secret；
        base_url (字符串, 非必填): 开放平台域名地址，默认值为 BASE_URL；
        timeout (数字, 非必填): 接口请求默认超时间，默认值为 DEFAULT_TIMEOUT；
    """

    def __init__(self, client_id, client_secret, base_url=BASE_URL,
                 timeout=DEFAULT_TIMEOUT):
        self.client_id = client_id
        self.client_secret = client_secret
        self.timeout = timeout
        self.base_url = base_url

    def get_authorization_url(self, redirect_uri, state=None, scopes=None):
        """获取用户授权页链接地址。

        Args:
            redirect_uri (str): 回调地址；
            state (str, 非必填): 第三方自定义信息，返回授权码时原样返回，
                                默认值为 None；
            scopes (list, 非必填): 权限范围，支持多个。默认值为 None；

        Returns:
            str: 用户授权页面地址；
        """
        params = {
            "client_id": self.client_id,
            "response_type": "code",
            "redirect_uri": redirect_uri
        }
        if scopes is not None:
            params['scope'] = " ".join(scopes)
        if state is not None:
            params["state"] = state
        qs = urlencode(params)
        return '?'.join((urljoin(self.base_url, '/oauth2/authorize'), qs))

    def exchange_authorization_code(self, code, redirect_uri):
        """用户授权码交换访问令牌

        Args:
            code (str): 用户授权后平台返回的授权码；
            redirect_uri (str): 回调地址；

        Returns:
            AuthorizationCodeToken: 与授权用户关联的访问令牌，同时包含有刷新令牌、
                                    过期时间等信息；
        """
        data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': GRANT_AUTHORIZATION_CODE,
            'redirect_uri': redirect_uri,
            'code': code
        }
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        result = request.post(
            '/oauth2/access_token', data=data, timeout=timeout)
        return models.AuthorizationCodeToken(self, result)

    def exchange_refresh_token(self, refresh_token):
        """刷新令牌 access_token

        Args:
            refresh_token (str): 授权码模式所获得的 refresh_token；

        Returns:
            Model: 新的令牌数据；
        """
        data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': 'refresh_token',
            'refresh_token': refresh_token
        }
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        result = request.post(
            '/oauth2/access_token', data=data, timeout=timeout)
        return models.AuthorizationCodeToken(self, result)

    def get_credentials_token(self):
        """客户端授权模式下获取访问令牌

        Returns:
            Model: 访问令牌，包含 access_token、过期时间等；
        """
        data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': GRANT_TYPE_CREDENTIALS
        }
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        result = request.post(
            '/oauth2/access_token', data=data, timeout=timeout)
        return models.ClientCredentialsToken(self, result)

    def get_api(self, access_token):
        """获取平台 API 调用客户端对象

        Args:
            access_token (str): 访问令牌；

        Returns:
            API: API 对象；
        """
        return API(
            access_token, base_url=self.base_url, timeout=self.timeout)


class API(object):
    """BGE 开放平台接口调用客户端

    Args:
        access_token (str): 访问令牌；
        base_url (字符串, 非必填): 开放平台域名地址，默认值为 BASE_URL；
        timeout (数字, 非必填): 接口请求默认超时间，默认值为 DEFAULT_TIMEOUT；
    """

    def __init__(self, access_token, base_url=BASE_URL,
                 timeout=DEFAULT_TIMEOUT):
        self.access_token = access_token
        self.base_url = base_url
        self.timeout = timeout

    def get_user(self, **params):
        """获取用户信息

        Returns:
            Model: 用户数据；
        """
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.get('/user', params=params, timeout=timeout)
        return models.Model(result)

    def get_variants(self, biosample_id, rsids, **params):
        """根据rsid查询变异位点信息

        Args:
            biosample_id (str): 生物样品编号；
            rsids (str): 多个 rs 编号，逗号分割（必填）；如：rs1229984；
                         最多一次提供100个；

        Returns:
            list: 变异位点信息；
        """
        params.update({
            'rsids': rsids,
            'biosample_id': biosample_id
        })
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.get('/variants', params=params, timeout=timeout)
        data = []
        for item in result:
            data.append(models.Model(item))
        return data

    def get_samples(self, biosample_ids=None, biosample_sites=None,
                    omics=None, project_ids=None, organisms=None,
                    data_availability=None, statuses=None,
                    next_page=None, limit=50, **kwargs):
        """获取样品列表

        授权码模式: 可通过本接口获取授权用户的样品；
        客户端模式: 可通过本接口获取客户端应用通过注册接口注册(或预先生成)的样品；

        Args:
            biosample_ids ([str], 非必填): 生物样品 id，逗号分割多个；
            biosample_sites ([str], 非必填): 采样部位，取值范围：1-15；
            omics ([str], 非必填): 所属组学，取值范围：1-2；
            project_ids ([str], 非必填): 所属项目，逗号分割多个；
            organisms ([str], 非必填): 样品生物体，取值范围：1-3；
            data_availability ([boolean], 非必填): 数据可用性；
            statuses ([str], 非必填): 数据状态，详情见 BGE 开放平台文档；
            next_page (int, 非必填): 要获取的页码，默认值为 None；
            limit (int, 非必填): 每页返回数量，默认值为 50；

        Returns:
            list: 样品列表；
        """
        params = {}
        params.update(kwargs)
        page = 1
        if next_page is not None:
            page -= 1
        params.update({
            'biosample_ids': biosample_ids,
            'biosample_sites': biosample_sites,
            'omics': omics,
            'project_ids': project_ids,
            'organisms': organisms,
            'data_availability': data_availability,
            'statuses': statuses,
            'page': page,
            'limit': limit
        })
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.get('/samples', params=params, timeout=timeout)
        result = models.Model(result)
        data = []
        for item in result['result']:
            data.append(models.Model(item))
        result['result'] = data
        return result

    def get_sample(self, biosample_id):
        """获取样品

        授权码模式: 可通过本接口获取授权用户的样品；
        客户端模式: 可通过本接口获取客户端应用通过注册接口注册(或预先生成)的样品；

        Args:
            biosample_id (str): 生物样品编号；

        Returns:
            Model: 样品数据；
        """
        url = '/samples/{}'.format(biosample_id)
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.get(url, timeout=timeout)
        return models.Model(result)

    def register_sample(self, external_sample_id, biosample_site,
                        project_id, **kwargs):
        """注册样品

        Args:
            external_sample_id (str): 外部生物样品id；
            biosample_site (int):  采样部位；
            project_id (str): 项目编号；
            **kwargs: 其他非必填数据，例：library_id="HWJBAYTGAA170328-18"；

        Returns:
            Model: 样品数据，包含生物样品编号；
        """
        data = {}
        data.update(kwargs)
        data.update({
            'external_sample_id': external_sample_id,
            'biosample_site': int(biosample_site),
            'project_id': project_id
        })
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.post(
            '/samples/register', data=data, timeout=timeout)
        return Model(result)

    def improve_sample(self, biosample_id, **kwargs):
        """补充样品中未被赋值的信息

        已赋值数据无法变更，否则接口报错；

        Args:
            biosample_id (str): 生物样品编号；
            **kwargs: 需要赋值的字段和值；
        """
        if not kwargs:
            # 无更新
            return
        data = {}
        data.update(kwargs)
        data['biosample_id'] = biosample_id
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        request.post(
            '/samples/improve', data=data, timeout=timeout)

    def get_taxon_abundance(self, biosample_id, taxon_ids=None,
                            next_page=None, limit=50, **params):
        """获取类群丰度

        Args:
            biosample_id (str): 生物样品编号；
            taxon_ids ([str], 非必填): BGE 物种编号，多个以逗号分割；
            next_page ([int], 非必填): 当前页码，默认值为 1，即首页；
            limit (int, 非必填): [description]. 默认值为 50；

        Returns:
            Model: 类群丰度数据详情；
        """
        params.update({
            'biosample_id': biosample_id,
            'taxon_id': taxon_ids,
            'limit': limit
        })
        page = 1
        if next_page is not None:
            page -= 1
        params['page'] = page
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result, pagination = request.get(
            '/microbiome/taxon_abundance', params=params, timeout=timeout)
        # TODO: upgrade in the future
        # 暂时特殊处理此接口，统一丰度数据的返回方式
        ret = {}
        ret['count'] = count = pagination['count']
        next_page = pagination['page'] + 1
        if count == 0:
            next_page = None
        ret['next_page'] = next_page
        ret['result'] = [models.Model(item) for item in result]
        return ret

    def get_func_abundance(self, biosample_id, catalog, ids=None, limit=50,
                           next_page=None, **params):
        """获取功能丰度

        Args:
            biosample_id (str): 生物样品编号；
            catalog (str): 目录标签，可选值为：go、ko、eggnog、pfam、kegg-pwy、
                           kegg-mdl、level4ec、metacyc-rxn、metacyc-pwy；
            ids (str, 非必填): BGE物种功能编号，多个值以逗号隔开；
            limit (int, 非必填): 一页返回数量，默认值为 50；
            next_page (str, 非必填): 下一页，用于获取下一页数据；

        Returns:
            Model: 功能丰度数据详情；
        """
        params.update({
            'biosample_id': biosample_id,
            'catalog': catalog,
            'ids': ids,
            'next_page': next_page,
            'limit': limit
        })
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.get(
            '/microbiome/func_abundance', params=params,
            timeout=timeout)
        result = models.Model(result)
        result['result'] = [models.Model(item) for item in result['result']]
        return result

    def get_gene_abundance(self, biosample_id, catalog, data_type, ids=None,
                           limit=None, next_page=None, **params):
        """获取基因丰度

        Args:
            biosample_id (str): 生物样品编号；
            catalog (str): 分类标签，可选值：IGC_9.9M、UniRef90_HUMAnN2_0.11；
            data_type (str): 返回数据类型，可选值：list、file；
            ids (str, 非必填): BGE 物种 IGC 基因编号，多个值以逗号分割，
                                    如：igc_50,igc_51；
            limit (int, 非必填): 一页最大返回数量，默认 50，最大值为 1000；
            next_page (str, 非必填): 接口返回的下一页参数；

        Returns:
            Model: 基因丰度数据详情；
        """
        params.update({
            'biosample_id': biosample_id,
            'catalog': catalog,
            'data_type': data_type,
            'ids': ids,
            'next_page': next_page,
            'limit': limit
        })
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.get(
            '/microbiome/gene_abundance', params=params, timeout=timeout)
        result = models.Model(result)
        result['result'] = [models.Model(item) for item in result['result']]
        return result

    def get_upload_token(self, **kwargs):
        """获取文件上传授权
        
        获取的授权仅包括当前目录（不含子目录）下的文件读、写权限；

        Returns:
            Model: 授权数据；
        """
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.post('/sts/token', data=kwargs, timeout=timeout)
        return models.Model(result)

    def upload(self, filename, file_or_string):
        token = self.get_upload_token()
        print(token)
        credentials = token.credentials
        destination = token.destination
        bucket_name = token.bucket
        endpoint = token.endpoint
        access_key_id = credentials['access_key_id']
        access_key_secret = credentials['access_key_secret']
        security_token = credentials['security_token']
        auth = oss2.StsAuth(
            access_key_id, access_key_secret, security_token)
        bucket = oss2.Bucket(auth, endpoint, bucket_name)
        object_name = '%s/%s' % (destination, filename)
        bucket.put_object(object_name, file_or_string)
        return object_name

    def get_download_url(self, object_name, region=None,
                         expiration_time=600, **kwargs):
        """获取阿里云OSS（对象存储）中的文件下载地址

        Args:
            object_name (str): OSS对象；
            region (str, 非必填): 区域（domestic、international），默认值为
                                 domestic；
            expiration_time (int, 非必填): 下载地址过期时间，默认值 600s；

        Returns:
            Model: 文件下载地址；
        """

        data = {}
        data.update(kwargs)
        data.update({
            'object_name': object_name,
            'region': region,
            'expiration_time': expiration_time
        })
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        result = request.post('/oss/sign_url', data=data, timeout=timeout)
        return models.Model(result)

    def invoke_model(self, model_id, **kwargs):
        """模型调用

        Args:
            model_id (str): 模型编号；
            **kwargs: 模型相关参数，由模型定义的参数决定；

        Returns:
            Model: 任务id，时间和状态；
        """
        params = {}
        params.update(kwargs)
        timeout = self.timeout
        request = HTTPRequest(self.base_url)
        request.set_authorization(self.access_token)
        model_url = '/model/{}'.format(model_id)
        result = request.get(model_url, params=params, timeout=timeout)
        return models.Model(result)



#-*- coding: utf-8 -*-

from bgesdk.error import APIError

import pytest


class TestOAuth2:

    def test_authorization_url_is_str(self, oauth2, redirect_url):
        """获取用户授权页面链接"""
        url = oauth2.get_authorization_url(redirect_url)
        assert isinstance(url, str)

    def test_invalid_code(self, oauth2):
        """错误的用户授权 code 和 redirect_uri"""
        with pytest.raises(APIError) as e:
            oauth2.exchange_authorization_code('code', 'http://test.cn')
        assert e.value.code == 400
        assert e.value.msg == u'非法请求: invalid_grant'

    def test_invalid_access_token(self, oauth2):
        """错误的 access_token"""
        access_token = 'demo'
        api = oauth2.get_api(access_token)
        with pytest.raises(APIError) as e:
            api.get_user()
        assert e.value.code == 403
        assert e.value.msg == u'access_token 已失效或错误'



import sys

if sys.version_info.major <= 2:
    reload(sys)
    sys.setdefaultencoding("UTF-8")

import setuptools

with open('requirements.txt') as f:
    install_requires = [line.strip() for line in f]

setuptools.setup(
    install_requires=install_requires
)



#-*- coding: utf-8 -*-

BASE_URL = 'https://api.bge.genomics.cn'

# 支持的 OAuth2 授权方式
GRANT_AUTHORIZATION_CODE = 'authorization_code'
GRANT_TYPE_CREDENTIALS = 'client_credentials'

HTTP_DEFAULT_TIMEOUT = 16
HTTP_MAX_RETRIES = 3